
import { BrowserRouter as Router } from 'react-router-dom';
import { act } from "react-dom/test-utils";
import { render, cleanup, fireEvent, screen ,waitFor} from '@testing-library/react';
import '@testing-library/jest-dom';

import mockConsole from 'jest-mock-console';

import {CycleCreate} from './create';
import CycleService from '../../services/cycle.service';
import UnitConverter from '../../utils/unit.converter';
import _ from 'lodash';
import CycleServiceMock from '../../__mocks__/cycle.service.data';

let saveCycleSpy, resourcesSpy, cycleResourceDefaultsSpy;
let restoreConsole;

beforeEach(() => {
    restoreConsole = mockConsole();
    setMockSpy();
});

afterEach(() => {
    restoreConsole();
    // cleanup on exiting
    clearMockSpy();
    cleanup();
});

/**
 * To set mock spy for Services that have API calls to the back end to fetch data
 */
const setMockSpy = (() => {    
    resourcesSpy = jest.spyOn(CycleService, 'getResources');
    resourcesSpy.mockImplementation(() => {
        return Promise.resolve(CycleServiceMock.resources);
    });   
    
    saveCycleSpy = jest.spyOn(CycleService, 'saveCycle');
    saveCycleSpy.mockImplementation((cycle ) => { 
       cycle.url = `http://localhost:3000/api/cycle/${cycle.name}`;
        return Promise.resolve(cycle)
    });
});

const clearMockSpy = (() => {
    saveCycleSpy.mockRestore();
});

it("renders without crashing with all back-end data loaded", async () => {
    console.log("renders without crashing with all back-end data loaded ------------------------");
    let content;
    await act(async () => {
        content = render(<Router><CycleCreate location={{pathname: '/cycle/create'}} unitMap={UnitConverter.resourceUnitMap} /></Router>);
    });
    expect(content.queryByText('Cycle - Add')).not.toBe(null);        // Page loaded successfully
    expect(content.queryAllByText('Add Resources').length).toBe(2);     // Resource Dropdown loaded successfully
    const spinButtons = content.queryAllByRole("spinbutton");
    expect(_.filter(spinButtons, {"id": "LOFAR Observing Time"})[0].value).toBe("0");
    fireEvent.click(screen.getAllByText("Add Resources")[0].parentElement.parentElement.parentElement.children[3]);
    expect(content.queryByText('CEP Storage')).toBeInTheDocument();         // Resources other than Default Resources listed in dropdown
    expect(content.queryByPlaceholderText('CEP Storage')).toBe(null);       // No resources other than Default Resources listed to get input
});

it("Save button disabled initially when no data entered", async () => {
    console.log("Save button disabled initially when no data entered -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><CycleCreate location={{pathname: '/cycle/create'}} unitMap={UnitConverter.resourceUnitMap} /></Router>);
    });
    expect(content.queryByTestId('save-btn')).toHaveAttribute("disabled");
});

it("Save button enabled when mandatory data entered", async () => {
    console.log("Save button enabled when mandatory data entered -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><CycleCreate location={{pathname: '/cycle/create'}} unitMap={UnitConverter.resourceUnitMap} /></Router>);
    });
    const nameInput = content.queryByTestId('name');
    const descInput = content.queryByTestId('description');

    
    // fireEvent.change(startInput, { target: { value: '2022-01-27' } });       
    // fireEvent.blur(startInput, { target: { value: '2022-01-27' } });   
    // expect(startInput.value).toBe("2022-01-27");
    // Select the cycle start date from calendar
    fireEvent.click(screen.getByTestId("start").children[1]);
    fireEvent.click(screen.getAllByText("1")[0]);    
    await new Promise((r) => setTimeout(r, 1000));

    // fireEvent.blur(stopInput,{ target: { value: '2022-01-28' } });
    // fireEvent.change(stopInput,{ target: { value: '2022-01-28' } });
    // expect(stopInput.value).toBe("2022-01-28");
    // Select the cycle stop date from calendar
    await act(async () =>  fireEvent.click(screen.getByTestId("stop").children[1]));
    await act(async () =>  fireEvent.click(screen.getAllByText("2")[0]));
    await act(async () =>  fireEvent.change(nameInput, { target: { value: 'Cycle-12' }} ));
    expect(nameInput.value).toBe("Cycle-12");
    await act(async () => {
        fireEvent.change(descInput, { target: { value: 'Cycle-12' } });
    });
    expect(descInput.value).toBe("Cycle-12");
  
    await new Promise((r) => setTimeout(r, 5000));
    expect(content.queryByTestId('save-btn').hasAttribute("disabled")).toBeFalsy();
});

 
it("save cycle with mandatory fields", async () => {
    console.log("save cycle -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><CycleCreate location={{pathname: '/cycle/create'}} unitMap={UnitConverter.resourceUnitMap} /></Router>);
    });

    let nameInput = content.queryByTestId('name');
    const descInput = content.queryByTestId('description');
    const createAnother = content.queryByTestId('createAnother');
    fireEvent.click(createAnother);
    fireEvent.change(nameInput, { target: { value: 'Cycle-12' } });
    expect(nameInput.value).toBe("Cycle-12");
    fireEvent.change(descInput, { target: { value: 'Cycle-12' } });
    expect(descInput.value).toBe("Cycle-12");

    // Select the cycle start date from calendar
    fireEvent.click(screen.getByTestId("start").children[1]);
    fireEvent.click(screen.getAllByText("1")[0]);    
    await new Promise((r) => setTimeout(r, 1000));

    // Select the cycle stop date from calendar
    fireEvent.click(screen.getByTestId("stop").children[1]);
    fireEvent.click(screen.getAllByText("2")[0]);


    await new Promise((r) => setTimeout(r, 500));
    expect(content.queryByTestId('save-btn').hasAttribute("disabled")).toBeFalsy();
    expect(content.queryByTestId('cycleId').value).toBe("");
    expect(content.queryByText("Success")).toBe(null);
    
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 1000));
    });
    console.log("Checking service calls..");
    expect(saveCycleSpy).toHaveBeenCalled();
    nameInput = content.queryByTestId('name');
    expect(nameInput.value).toBe("");
    expect(content.queryByTestId('save-btn')).toHaveAttribute("disabled");
});

it("save cycle with default resources", async () => {
    console.log("save cycle with default resources -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><CycleCreate location={{pathname: '/cycle/create'}} unitMap={UnitConverter.resourceUnitMap} /></Router>);
    });

    let nameInput = content.queryByTestId('name');
    const descInput = content.queryByTestId('description');
    const spinButtons = content.queryAllByRole("spinbutton");
    const createAnother = content.queryByTestId('createAnother');
    fireEvent.click(createAnother);
    fireEvent.change(nameInput, { target: { value: 'Cycle-12' } });
    expect(nameInput.value).toBe("Cycle-12");
    fireEvent.change(descInput, { target: { value: 'Cycle-12' } });
    expect(descInput.value).toBe("Cycle-12");

    // Select the cycle start date from calendar
    fireEvent.click(screen.getByTestId("start").children[1]);
    fireEvent.click(screen.getAllByText("1")[0]);    
    await new Promise((r) => setTimeout(r, 1000));

    // Select the cycle stop date from calendar
    fireEvent.click(screen.getByTestId("stop").children[1]);
    fireEvent.click(screen.getAllByText("2")[0]);

    await new Promise((r) => setTimeout(r, 500));
    expect(content.queryByTestId('save-btn').hasAttribute("disabled")).toBeFalsy();
    expect(content.queryByTestId('cycleId').value).toBe("");
    expect(content.queryByText("Success")).toBe(null);
    
    const lofarObsTimeInput = _.filter(spinButtons, {"id": "LOFAR Observing Time"})[0];
    fireEvent.change(lofarObsTimeInput, { target: { value: 10 } });
    expect(lofarObsTimeInput.value).toBe('10');

    const cepProcTimeInput = _.filter(spinButtons, {"id": "CEP Processing Time"})[0];
    fireEvent.change(cepProcTimeInput, { target: { value: 5 } });
    expect(cepProcTimeInput.value).toBe('5');
    
    const ltaStorageInput = _.filter(spinButtons, {"id": "LTA Storage"})[0];
    fireEvent.change(ltaStorageInput, { target: { value: 2 } });
    expect(ltaStorageInput.value).toBe('2'); 
    
    const lofarObsTimeAInput = _.filter(spinButtons, {"id": "LOFAR Observing Time prio A"})[0];
    fireEvent.change(lofarObsTimeAInput, { target: { value: 15 } });
    expect(lofarObsTimeAInput.value).toBe('15');
    
    const lofarObsTimeBInput = _.filter(spinButtons, {"id": "LOFAR Observing Time prio B"})[0];
    fireEvent.change(lofarObsTimeBInput, { target: { value: 20 } });
    expect(lofarObsTimeBInput.value).toBe('20');
   
    fireEvent.change(lofarObsTimeBInput, { target: { value: '' } });
    expect(lofarObsTimeBInput.value).toBe('');
    const lofarSupTimeInput = _.filter(spinButtons, {"id": "LOFAR Support Time"})[0];
    fireEvent.change(lofarSupTimeInput, { target: { value: 25 } });
    expect(lofarSupTimeInput.value).toBe('25');
    
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 1000));
    });
    console.log("Checking service calls..");
    expect(saveCycleSpy).toHaveBeenCalled();
    nameInput = content.queryByTestId('name');
    expect(nameInput.value).toBe("");
    expect(content.queryByTestId('save-btn')).toHaveAttribute("disabled");
});


it("save cycle with added resources", async () => {
    console.log("save cycle with added resources -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><CycleCreate location={{pathname: '/cycle/create'}} unitMap={UnitConverter.resourceUnitMap} /></Router>);
    });

    let nameInput = content.queryByTestId('name');
    const descInput = content.queryByTestId('description');
    let spinButtons = content.queryAllByRole("spinbutton");
    const createAnother = content.queryByTestId('createAnother');
    fireEvent.click(createAnother);
    fireEvent.change(nameInput, { target: { value: 'Cycle-12' } });
    expect(nameInput.value).toBe("Cycle-12");
    fireEvent.change(descInput, { target: { value: 'Cycle-12' } });
    expect(descInput.value).toBe("Cycle-12");

    // Select the cycle start date from calendar
    fireEvent.click(screen.getByTestId("start").children[1]);
    fireEvent.click(screen.getAllByText("1")[0]);    
    await new Promise((r) => setTimeout(r, 1000));

    // Select the cycle stop date from calendar
    fireEvent.click(screen.getByTestId("stop").children[1]);
    fireEvent.click(screen.getAllByText("2")[0]);

    await new Promise((r) => setTimeout(r, 500));

    expect(content.queryByTestId('save-btn').hasAttribute("disabled")).toBeFalsy();
    expect(content.queryByTestId('cycleId').value).toBe("");
    expect(content.queryByText("Success")).toBe(null);
    
    const lofarObsTimeInput = _.filter(spinButtons, {"id": "LOFAR Observing Time"})[0];
    fireEvent.change(lofarObsTimeInput, { target: { value: 10 } });
    expect(lofarObsTimeInput.value).toBe('10');

    const cepProcTimeInput = _.filter(spinButtons, {"id": "CEP Processing Time"})[0];
    fireEvent.change(cepProcTimeInput, { target: { value: 5 } });
    expect(cepProcTimeInput.value).toBe('5');
    
    const ltaStorageInput = _.filter(spinButtons, {"id": "LTA Storage"})[0];
    fireEvent.change(ltaStorageInput, { target: { value: 2 } });
    expect(ltaStorageInput.value).toBe('2'); 
    
    const lofarObsTimeAInput = _.filter(spinButtons, {"id": "LOFAR Observing Time prio A"})[0];
    fireEvent.change(lofarObsTimeAInput, { target: { value: 15 } });
    expect(lofarObsTimeAInput.value).toBe('15');
    
    const lofarObsTimeBInput = _.filter(spinButtons, {"id": "LOFAR Observing Time prio B"})[0];
    fireEvent.change(lofarObsTimeBInput, { target: { value: 20 } });
    expect(lofarObsTimeBInput.value).toBe('20');
   
    const lofarSupTimeInput = _.filter(spinButtons, {"id": "LOFAR Support Time"})[0];
    fireEvent.change(lofarSupTimeInput, { target: { value: 25 } });
    expect(lofarSupTimeInput.value).toBe('25');
    
    // Before selecting New Resource
    expect(content.queryAllByText('Add Resources').length).toBe(2);
    expect(content.queryAllByText('CEP Storage').length).toBe(0);
    expect(content.queryByPlaceholderText('CEP Storage')).toBe(null);
    const addResourcesdropdownicon = screen.getAllByText("Add Resources")[0].parentElement.parentElement.parentElement.children[3]; // ..
    fireEvent.click(addResourcesdropdownicon);
    const addResourceInputCEPSTorage = screen.getAllByText("CEP Storage")[0] ;
    await act(() =>fireEvent.click(addResourceInputCEPSTorage));
    await new Promise((r) => setTimeout(r, 500));

    // After selecting New Resource
    await waitFor (() =>expect(content.queryAllByText('Add Resources').length==1)); 

    expect(content.queryAllByText('CEP Storage').length==3);

    const addResourceBtn = content.queryByTestId('add_res_btn');
    await act(() =>fireEvent.click(addResourceBtn));
    await new Promise((r) => setTimeout(r, 500));

    await waitFor (() =>expect(content.queryAllByText('Add Resources').length).toBe(2));
    
    spinButtons = content.queryAllByRole("spinbutton");
    const newResourceInput =  _.filter(spinButtons, {"id": "CEP Storage"})[0];
    fireEvent.change(newResourceInput, { target: { value: 30 } });
    expect(newResourceInput.value).toBe('30');
    
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 1000));
    });
    console.log("Checking service calls..");
    expect(saveCycleSpy).toHaveBeenCalled();
    nameInput = content.queryByTestId('name');
    expect(nameInput.value).toBe("");
    expect(content.queryByTestId('save-btn')).toHaveAttribute("disabled");
});




it("remove default resource and added resource", async () => {
    console.log("remove default resource and added resource -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><CycleCreate location={{pathname: '/cycle/create'}} unitMap={UnitConverter.resourceUnitMap} /></Router>);
    });

    // Before selecting New Resource
    expect(content.queryAllByText('Add Resources').length).toBe(2);
    expect(content.queryByPlaceholderText('Support hours')).toBe(null);
    fireEvent.click(screen.getAllByText("Add Resources")[0].parentElement.parentElement.parentElement.children[3]);
    const addResourceInput = screen.getAllByText("CEP Storage")[0];
    fireEvent.click(addResourceInput);
    // After selecting New Resource
    
    await waitFor (() =>expect(content.queryAllByText('Add Resources').length==1)); 
    await waitFor (() =>expect(content.queryAllByText('CEP Storage').length==3)); 
    expect(content.queryAllByText('CEP Storage').length).toBe(3);

    const addResourceBtn = content.queryByTestId('add_res_btn');
    fireEvent.click(addResourceBtn);
    expect(content.queryAllByText('Add Resources').length).toBe(2);
    const spinButtons = content.queryAllByRole("spinbutton");
    expect(_.filter(spinButtons, {"id": "CEP Storage"})[0]).not.toBe(null);

    expect(content.queryByTestId('CEP Processing Time-btn')).not.toBe(null);
    const removeDefResBtn = content.queryByTestId('CEP Processing Time-btn');
    await act(async () => {
        fireEvent.click(removeDefResBtn);
    });
    expect(_.filter(spinButtons, {"id": "CEP Processing Time"}).length).toBe(0);
    expect(content.queryByTestId('CEP Processing Time-btn')).toBe(null);

    const removeResourceBtn = content.queryByTestId('CEP Storage-btn');
    fireEvent.click(removeResourceBtn);
    expect(content.queryAllByText('Add Resources').length).toBe(2);
    await new Promise((r) => setTimeout(r, 200));
    expect(content.queryAllByText('CEP Storage').length).toBe(0);
}

);

