import MathUtility from '../../../utils/math.utility';
import _ from 'lodash';
import UnitConverter from "../../../utils/unit.converter";
import moment from 'moment';

const ReportBusiness = {

     REPORT_VARIABLE_MAP : {
        "Intro": "report_intro",
        "Overview": "overview",
        "TelescopeTime": "telescope_time_distribution",
        "AverageEfficiency": "average_efficiency",
        "CompletionLevel": "completion_level",
        "CategoryObservation": "observation_durations_per_category",
        "WeeklyEfficiency": "weekly_efficiency",
        "DataIngestedPerSite": "data_ingested_per_site_and_category",
        "DataIngestedPerCategory": "data_ingested_per_site_and_category",
        "ProjectSummary": "projects_summary",
        "ILT_LocalUsage": "usage_mode",
        "Failures": "failures"
    },
    
     REGULAR_DDT_COMM : ['REGULAR', 'DDT', 'COMMISSIONING', 'USER_SHARED_SUPPORT','UNASSIGNED'],
     isDebugLogging :false,    

     /**
     * To extract data required for the sub report component from the cycle report API response data.
     * @param {String} reportVariable - sub report variable from the cycle report API response
     * @returns Object - Extracted object for the respective sub report component
     */
      getReportData(reportVariable,cyclesReportData) {

        let CurrentCyclesReportData = cyclesReportData
        let reportData = _.map(CurrentCyclesReportData, data => {
            let repData = { cycle: data.cycle };
            
            repData[reportVariable] = data.data[reportVariable];
            if (reportVariable === 'overview') {
                repData['completionLevel'] = data.data['completion_level'];
                repData['projects_summary']  = data.data['projects_summary'];
            }
            return repData;
        });
        if (this.isDebugLogging) console.log("Returning for" + reportVariable,reportData,CurrentCyclesReportData);
        return reportData;
    },



    generateAVGEffencyTableData  (forTable, data) {
        const reportData = [];
        _.map(data, repData => {
            reportData.push({
                cycle: repData.cycle,
                efficiency: repData?.average_efficiency?.efficiency ? repData?.average_efficiency?.efficiency * 100 : null,
                target: repData?.average_efficiency?.target ? repData?.average_efficiency?.target * 100 : null
            });
        });
        if (forTable && reportData.length > 0) {
            let efficiency = MathUtility.getAverage(_.map(reportData, 'efficiency'));
            let target = MathUtility.getAverage(_.map(reportData, 'target'));
            reportData.push({
                cycle: 'Average',
                efficiency: efficiency,
                target: target
            });
        }
        return reportData;
    },

    generateCategoryData  (forTable, data) {
        let reportData = [];
        const dataConversionFactor = UnitConverter.resourceUnitMap["bytes"].conversionFactor;
        data.map(repData => {
            let categoryData = {}
            const cycleData = repData.data_ingested_per_site_and_category?.categories;
            categoryData['cycle'] = repData.cycle;
            // TODO: Update field names and values once API is complete
            categoryData['rawIF'] = cycleData?.["Interferometric Observation"].size__sum / dataConversionFactor
            categoryData['rawBF'] = cycleData?.["Beamformed Observation"].size__sum / dataConversionFactor
            categoryData['preprocIF'] = cycleData?.["Preprocessing Pipeline"].size__sum / dataConversionFactor
            categoryData['pulpBF'] = cycleData?.["Pulsar Pipeline"].size__sum / dataConversionFactor
            reportData.push(categoryData);
            return repData;
        });

        if (forTable) {
            reportData.push({
                cycle: 'Average',
                rawIF: MathUtility.getAverage(_.map(reportData, 'rawIF'), 2),
                rawBF: MathUtility.getAverage(_.map(reportData, 'rawBF'), 2),
                preprocIF: MathUtility.getAverage(_.map(reportData, 'preprocIF'), 2),
                pulpBF: MathUtility.getAverage(_.map(reportData, 'pulpBF'), 2),
            });
        }

        reportData.forEach(entry => {
            entry.rawIF = parseFloat(entry.rawIF).toFixed(2);
            entry.rawBF = parseFloat(entry.rawBF).toFixed(2);
            entry.preprocIF = parseFloat(entry.preprocIF).toFixed(2);
            entry.pulpBF = parseFloat(entry.pulpBF).toFixed(2);
          });

        return reportData;
    },


    generateCategoryObservations(isTableData, data) {
        let reportData = [];
        const timeConversionFactor = UnitConverter.resourceUnitMap["time"].conversionFactor;
        data.map(repData => {
            let cycleData = {}
            cycleData['cycle'] = repData.cycle;
            cycleData['prio_A'] = repData.observation_durations_per_category?.total_duration_successful_A / timeConversionFactor
            cycleData['prio_B'] = repData.observation_durations_per_category?.total_duration_successful_B / timeConversionFactor
            cycleData['ddt_com_rep'] = repData.observation_durations_per_category?.["DDT Com Rep"] / timeConversionFactor
            cycleData['unavailable'] = repData.observation_durations_per_category?.["System Unavailability"] / timeConversionFactor
            cycleData['tmss_com'] = repData.observation_durations_per_category?.["TMSS Com"] / timeConversionFactor
            cycleData['failed'] = repData.observation_durations_per_category?.total_duration_failed / timeConversionFactor
            cycleData['idle'] = repData.observation_durations_per_category?.total_duration_idle / timeConversionFactor
            reportData.push(cycleData);
        });

        if (isTableData && reportData.length > 0) {
            reportData.push({
                cycle: 'Average',
                prio_A: MathUtility.getAverage(_.map(reportData, 'prio_A')),
                prio_B: MathUtility.getAverage(_.map(reportData, 'prio_B')),
                ddt_com_rep: MathUtility.getAverage(_.map(reportData, 'ddt_com_rep')),
                unavailable: MathUtility.getAverage(_.map(reportData, 'unavailable')),
                failed: MathUtility.getAverage(_.map(reportData, 'failed')),
                idle: MathUtility.getAverage(_.map(reportData, 'idle'))
            });
        }


        
        reportData.forEach(entry => {
            entry.prio_A = parseFloat(entry.prio_A).toFixed(2);
            entry.prio_B = parseFloat(entry.prio_B).toFixed(2);
            entry.ddt_com_rep = parseFloat(entry.ddt_com_rep).toFixed(2);
            entry.unavailable = parseFloat(entry.unavailable).toFixed(2);
            entry.failed = parseFloat(entry.failed).toFixed(2);
            entry.idle = parseFloat(entry.idle).toFixed(2);

          });

        return reportData;
    },


    generateCycleCompletion(forTable, data) {
        const reportData = [];
        _.map(data, repData => {
            const completionLevel = repData.completion_level;
            reportData.push({
                cycle: repData.cycle,
                performed: completionLevel?.total ? parseInt((completionLevel.succeeded / completionLevel?.total * 100).toFixed(0)) : null,
                prognosis: completionLevel?.total ? parseInt((completionLevel.prognosis / completionLevel?.total * 100).toFixed(0)) : null,
                // prognosis: completionLevel.prognosis,
                target: repData.completion_level?.target ? repData.completion_level?.target * 100 : null
            });
        });
        if (forTable && reportData.length > 0) {
            reportData.push({
                cycle: 'Average',
                performed: MathUtility.getAverage(_.map(reportData, 'performed')),
                prognosis: MathUtility.getAverage(_.map(reportData, 'prognosis')),
                target: MathUtility.getAverage(_.map(reportData, 'target'))
            });
        }
        return reportData;
    },

    /**
   * Function to format data list after calculations and include average if require for table or chart.
   * @param {Boolean} forTable - flag to incluce average value if the data is for table component.
   * @returns Array
   */
    generateFailureRate(forTable, data) {
        const timeConversionFactor = UnitConverter.resourceUnitMap.time.conversionFactor;
        // Assuming data structure
        // [{cycle: 'Cycle 15', failures: [{month: "2020-06-01", duration: 1200}, {month: "2020-07-01", duration: 1300},....]}]
        let reportData = { chartData: [], monthTableData: [], cycleTableData: [] };
        _.map(data, repData => {
            const monthlyFailures = repData.failures?.months;
            let cycleMonthlyFailures = []
            if (monthlyFailures !== undefined)
                for (const failureData of monthlyFailures) {
                    cycleMonthlyFailures.push({
                        cycle: repData.cycle,
                        date: failureData.month,
                        month: moment(failureData.month).format("YYYY-MM"),
                        duration: failureData.total_failed ? (failureData.total_failed / timeConversionFactor).toFixed(2) : null
                    });
                }
            reportData.chartData = reportData.chartData.concat(cycleMonthlyFailures);
            reportData.cycleTableData.push({
                cycle: repData.cycle,
                period: _.minBy(cycleMonthlyFailures, 'date') ?
                    `${(_.minBy(cycleMonthlyFailures, 'date')).month} -  ${(_.maxBy(cycleMonthlyFailures, 'date')).month}` : '',
                duration: ((_.sumBy(monthlyFailures, 'total_failed')) / timeConversionFactor).toFixed(2)
            });
        });
        reportData.monthTableData = reportData.chartData;
       
        return reportData;
    },


    /**Function to format data as required for the reports
 * 
 * @param {Array} data  
 */
    generateIltLocalUsage(data) {
        let reportData = [];
        if (data==null || data.length==0) { return  reportData}
        const timeConversionFactor = UnitConverter.resourceUnitMap["time"].conversionFactor;
        data.map(iltCycleData => {
            let iltReportData = {};
            iltReportData['cycle'] = iltCycleData.cycle;
            iltReportData['observing'] = (iltCycleData.usage_mode?.['ILT mode'].observing / timeConversionFactor).toFixed(2);
            iltReportData['idle'] = (iltCycleData.usage_mode?.['ILT mode']['idle/test'] / timeConversionFactor).toFixed(2);
            iltReportData['local_mode'] = (iltCycleData.usage_mode?.['stand-alone mode'].total / timeConversionFactor).toFixed(2);
            iltReportData['ilt_total'] = (iltCycleData.usage_mode?.['ILT mode'].total / timeConversionFactor).toFixed(2);
            iltReportData['ilt_mode_dutch'] = null;
            reportData.push(iltReportData);
        });
        if (reportData.length > 0) {
            reportData.push({
                cycle: 'Average',
                observing: MathUtility.getAverage(_.map(reportData, 'observing')),
                idle: MathUtility.getAverage(_.map(reportData, 'idle')),
                local_mode: MathUtility.getAverage(_.map(reportData, 'local_mode')),
                ilt_total: MathUtility.getAverage(_.map(reportData, 'ilt_total')),
                ilt_mode_dutch: MathUtility.getAverage(_.map(reportData, 'ilt_mode_dutch'))
            });
        }
        return  reportData
    },
    /**
         * Returns the overview of resources used in cycle(s) formatted to display in overview table.
         * @returns Array of used resources of cycle
         */
    generateOverView(data) {
        let reportData = [];
        const timeConversionFactor = UnitConverter.resourceUnitMap["time"].conversionFactor;
        const dataConversionFactor = UnitConverter.resourceUnitMap["bytes"].conversionFactor;
        data.map(repData => {
            const cycleProjects = repData.projects_summary;
            let projectData = {}
            projectData['cycle'] = repData.cycle;
            projectData['observingTime'] = (_.sumBy(cycleProjects, (projectSummary) => { return projectSummary.durations.total_observed_succeeded }) / timeConversionFactor).toFixed(2);
            projectData['noOfTriggers'] = _.sumBy(cycleProjects, (projectSummary) => { return projectSummary.nr_of_used_triggers });
            projectData['processingTime'] = (_.sumBy(cycleProjects, (projectSummary) => { return projectSummary.durations.total_CPU_time_used }) / timeConversionFactor).toFixed(2);
            projectData['ltaStorage'] = (_.sumBy(cycleProjects, (projectSummary) => { return projectSummary.LTA_dataproducts.size || 0 }) / dataConversionFactor).toFixed(2);
            projectData['prioSuccessTime'] = (_.sumBy(cycleProjects, (projectSummary) => { return projectSummary.durations.total_observed_succeeded_A }) / timeConversionFactor).toFixed(2);
            projectData['failedTime'] = (_.sumBy(cycleProjects, (projectSummary) => { return projectSummary.durations.total_observed_failed }) / timeConversionFactor).toFixed(2);
            projectData['completionLevel'] = repData?.completionLevel?.succeeded_perc || 0;
            reportData.push(projectData);
        });

        return reportData;
    },

    /**
     * Returns the projects summary of the cycle(s) formatted to display in summary table.
     * @returns Array of Cycle Projects list
     */
    generateProjectSummary  (data) {
        let reportData = [];
        const timeConversionFactor = UnitConverter.resourceUnitMap["time"].conversionFactor;
        const dataConversionFactor = UnitConverter.resourceUnitMap["bytes"].conversionFactor;
        if (data==null) return reportData
        data.map(repData => {
            const cycleProjects = repData.projects_summary;

            if (cycleProjects!==undefined)
            for (const projData of cycleProjects) {
                let projectData = {}
                projectData['cycle'] = repData.cycle;
                projectData['project'] = projData.project;
                projectData['durationObserved'] = (projData.durations.total_observed / timeConversionFactor).toFixed(2);
                projectData['durationProcessed'] = (projData.durations.total_CPU_time_used / timeConversionFactor).toFixed(2);
                const saraData = projData.data_ingested_per_site ? _.find(projData.data_ingested_per_site, { "cluster_name": "Sara" }) : null;
                projectData['dataToSara'] = ((saraData ? saraData.size : 0) / dataConversionFactor).toFixed(2);
                const juelichData = projData.data_ingested_per_site ? _.find(projData.data_ingested_per_site, { "cluster_name": "Juelich" }) : null;
                projectData['dataToJuelich'] = ((juelichData ? juelichData.size : 0) / dataConversionFactor).toFixed(2);
                const poznanData = projData.data_ingested_per_site ? _.find(projData.data_ingested_per_site, { "cluster_name": "Poznan" }) : null;
                projectData['dataToPoznan'] = ((poznanData ? poznanData.size : 0) / dataConversionFactor).toFixed(2);
                reportData.push(projectData);
            }
            
        });
        return reportData;
    },
     generateReportSiteData (data) {
        let reportData = [];
        const dataConversionFactor = UnitConverter.resourceUnitMap["bytes"].conversionFactor;
        data.map(repData => {
            let siteData = {}
            const cycleData = repData.data_ingested_per_site_and_category?.sites;
            siteData['cycle'] = repData.cycle;
            const saraData = _.find(cycleData, { "cluster_name": "Sara" });
            siteData['sara'] = ((saraData ? saraData.size : 0) / dataConversionFactor).toFixed(2);
            const juelichData = _.find(cycleData, { "cluster_name": "Juelich" });
            siteData['juelich'] = ((juelichData ? juelichData.size : 0) / dataConversionFactor).toFixed(2);
            const poznanData = _.find(cycleData, { "cluster_name": "Poznan" });
            siteData['poznan'] = ((poznanData ? poznanData.size : 0) / dataConversionFactor).toFixed(2);
            reportData.push(siteData);
        });

        return reportData;
    },

        /**
     * Function for formating data as required for the Chart and Table
     * @param {Array} data 
     */
        generateTelescopeTime(data) {
            let reportData =[];
            data.map(cycleData => {
                let cycleReportData = {};
                let cycleTelescopeTime = cycleData.telescope_time_distribution;
                if (cycleTelescopeTime!==undefined) {
                    let totalCycleDuration = _.sumBy(Object.values(cycleTelescopeTime), ttdValue => ttdValue.durations.total);
                    cycleReportData['cycle'] = cycleData.cycle;
                    if(totalCycleDuration > 0){
                        cycleReportData['reg_ddt_comm'] = this.calculateReportData(_.pick(cycleTelescopeTime,this.REGULAR_DDT_COMM), 'reg_ddt_comm', totalCycleDuration);                                       
                        cycleReportData['filler'] = this.calculateReportData(_.pick(cycleTelescopeTime,['FILLER']), 'filler', totalCycleDuration);               
                        cycleReportData['failed'] = this.calculateReportData(cycleTelescopeTime, 'failed', totalCycleDuration);
                        cycleReportData['idle'] = this.calculateReportData(cycleTelescopeTime, 'idle', totalCycleDuration);
                    }
                    reportData.push(cycleReportData);
                }
            });
            if (reportData.length>0) {
                reportData.push({cycle: 'Average',
                                    reg_ddt_comm: MathUtility.getAverage(_.map(reportData, 'reg_ddt_comm')), 
                                    filler: MathUtility.getAverage(_.map(reportData, 'filler')),
                                    failed: MathUtility.getAverage(_.map(reportData, 'failed')), 
                                    idle: MathUtility.getAverage(_.map(reportData, 'idle'))
                                });
            }
           return reportData;
        },

        generateWeeklyEfficiency(data) {
            let reportData = {chartData: [], tableData: []};
            _.map(data, repData=> {
                const weekly_efficiency = repData?.weekly_efficiency;
                let index = 1;
    
                if (weekly_efficiency)
                for (const weekData of weekly_efficiency.weeks) {
                    
                    reportData.chartData.push({   cycle: repData.cycle,
                                        weekDate: weekData.week,
                                        weekNumber: moment(weekData.week).week(),
                                        'Week Index': 'Week - ' + index,
                                        efficiency: parseInt((weekData.efficiency * 100).toFixed(0))
                                    })
                    index++;
                }
            });
            const cycles = _.keys(_.groupBy(reportData.chartData, 'cycle'));
            reportData.cycles =cycles;
                const weekGroupData = _.groupBy(reportData.chartData, 'Week Index');
                for (const week of _.keys(weekGroupData)) {
                    const weekData = weekGroupData[week];
                    let groupData = {'Week Index': week };
                    for (const cycle of weekData) {
                        groupData[cycle.cycle + " Efficiency %"] = cycle.efficiency;
                    }
                    reportData.tableData.push(groupData);
                }
                if (reportData.tableData.length > 0) {
                    let averageData = {'Week Index': 'Average'};
                    for (const column of (_.keys(reportData.tableData[0]))) {
                        if (column !== 'Week Index') {
                            averageData[column] = MathUtility.getAverage(_.map(reportData.tableData, column));
                        }
                    }
                    reportData.tableData.push(averageData);
                }

            // }
            return reportData;
        },

        generateDataFor(isTableData,cyclesReportData) {
            let reportData = {}
           reportData["AverageEfficiency"] = this.generateAVGEffencyTableData(isTableData,this.getReportData(this.REPORT_VARIABLE_MAP["AverageEfficiency"],cyclesReportData))
           reportData["DataIngestedPerCategory"] = this.generateCategoryData(isTableData,this.getReportData(this.REPORT_VARIABLE_MAP["DataIngestedPerCategory"],cyclesReportData))
           reportData["CategoryObservation"] = this.generateCategoryObservations(isTableData,this.getReportData(this.REPORT_VARIABLE_MAP["CategoryObservation"],cyclesReportData))
           reportData["CompletionLevel"] = this.generateCycleCompletion(isTableData,this.getReportData(this.REPORT_VARIABLE_MAP["CompletionLevel"],cyclesReportData))
           reportData["Failures"] = this.generateFailureRate(isTableData,this.getReportData(this.REPORT_VARIABLE_MAP["Failures"],cyclesReportData))
           reportData["ILT_LocalUsage"] = this.generateIltLocalUsage(this.getReportData(this.REPORT_VARIABLE_MAP["ILT_LocalUsage"],cyclesReportData))
           reportData["Overview"] = this.generateOverView(this.getReportData(this.REPORT_VARIABLE_MAP["Overview"],cyclesReportData))
           reportData["ProjectSummary"] = this.generateProjectSummary(this.getReportData(this.REPORT_VARIABLE_MAP["ProjectSummary"],cyclesReportData))
           reportData["DataIngestedPerSite"] = this.generateReportSiteData(this.getReportData(this.REPORT_VARIABLE_MAP["DataIngestedPerSite"],cyclesReportData))
           reportData["TelescopeTime"] = this.generateTelescopeTime(this.getReportData(this.REPORT_VARIABLE_MAP["TelescopeTime"],cyclesReportData))
           reportData["WeeklyEfficiency"] = this.generateWeeklyEfficiency(this.getReportData(this.REPORT_VARIABLE_MAP["WeeklyEfficiency"],cyclesReportData))
            // showhow data['date']; has to be eeleted  from monthTableData ?
            return reportData;
        },
        generateVisualData(cyclesReportData) {
            return  this.generateDataFor(true,cyclesReportData) 
        },
    
        generateExportData(cyclesReportData){
            return  this.generateDataFor(false,cyclesReportData) 
        },
          /**
     * Function for calculating the date for the report
     * @param {Object} groupData Formated data required for calculations
     * @param {String} groupType 
     * @param {Float} totalCycle Sum of all the totals for percentage calculations
     * @returns Float
     */
     calculateReportData  (groupData, groupType, totalCycle)  {
        let groupArr = Object.values(groupData);
        let percent;
        if (groupType === 'reg_ddt_comm' || groupType === 'filler')
            percent = parseFloat(((_.sumBy(groupArr, cal => cal.durations.succeeded) / totalCycle) * 100).toFixed(0));
        if (groupType === 'failed')
            percent = parseFloat(((_.sumBy(groupArr, cal => cal.durations.failed) / totalCycle) * 100).toFixed(0));
        if (groupType === 'idle')
            percent = parseFloat((((_.sumBy(groupArr, cal => cal.durations.idle) + groupData['TEST'].durations.succeeded) / totalCycle) * 100).toFixed(0));
       return percent > 0 ? percent : null;
    }
      
}





export default ReportBusiness;