import React, { useState, useEffect, useRef, forwardRef } from 'react';
import moment from 'moment';
import _ from 'lodash';
import Papa from "papaparse";
import jsPDF from 'jspdf';
import html2canvas from 'html2canvas';

import { useReactToPrint } from 'react-to-print';
import { AutoComplete } from 'primereact/autocomplete';
import { Calendar } from 'primereact/calendar';
import { Checkbox } from 'primereact/checkbox';
import { Button } from 'primereact/button';
import { ProgressBar } from 'primereact/progressbar';
import { AppLoader } from '../../../layout/components/AppLoader';
import { appGrowl } from '../../../layout/components/AppGrowl';
import { CustomDialog } from '../../../layout/components/CustomDialog';
import CycleService from '../../../services/cycle.service';
import ReportService from '../../../services/report.service';
import CycleReportContent from './report.content'
import ReportBusiness from './report.business'


/**
 * Main component for Cycle Report
 */
const CycleReportMain = forwardRef(() => {

    const [pageLoading, setPageLoading] = useState(true);
    const [selectedCycles, setSelectedCycles] = useState([]); // Used by the Autocomplete field
    const [reportCycles, setReportCycles] = useState([]);  // Used to generate the report for selected cycles
    const [reportPeriod, setReportPeriod] = useState([]);  // Period of reporting
    const [dialog, setDialog] = useState([]);
    const [cycles, setCycles] = useState([]);
    const [suggestedCycles, setSuggestedCycles] = useState();
    const [isDownloading, setIsDownloading] = useState(false);
    const [reportNames, setReportNames] = useState([]);
    let shadowReportNames = [];
    const [csvDlgVisible, setCsvDlgVisible] = useState();
    const [isLoading, setIsLoading] = useState(false);
    const [cyclesReportData, setCyclesReportData] = useState({});
    const [visualReportData, setVisualReportData] = useState({});

    const [progressDots, setProgressDots] = useState(".");
    const reportComponentRef = useRef();




    const handlePrint = useReactToPrint({
        content: () => reportComponentRef.current,
    });

    const FetchAllCyclesWithQuota = () => {
        CycleService.getAllCyclesWithQuota().then(
            cycles => {
                setCycles(cycles);
                setPageLoading(false);
            });
    }

    useEffect(() => {
        FetchAllCyclesWithQuota();
    }, []);



    /**
     * Function passed to AutoComplete component to search and list
     * @param {*} event 
     */
    const searchCycles = (event) => {
        setTimeout(() => {
            let suggestedCycles;
            if (!event.query.trim().length) {
                suggestedCycles = [...cycles];
            }
            else {
                suggestedCycles = cycles.filter((cycle) => {
                    // For matching the start of the cycle with the search text
                    // return cycle.name.toLowerCase().startsWith(event.query.toLowerCase());
                    // For values containing the search text
                    return cycle.name.toLowerCase().indexOf(event.query.toLowerCase()) >= 0;
                });
            }
            setSuggestedCycles(suggestedCycles);
        }, 250);
    }

    /**
     * Function to set the selected cycles from the AutoComplete component
     * @param {Array} cycles 
     */
    const selectCycles = (cycles) => {
        let newReportPeriod = [];
        if (cycles.length > 0) {
            let cycleDates = [];
            // Get all cycle dates
            for (const cycle of cycles) {
                cycleDates.push(moment(cycle.start));
                cycleDates.push(moment(cycle.stop));
            }
            // Get the minimum and maximum date from the cycle dates and set as the report period
            let minCycleDate = _.min(cycleDates);
            let maxCycleDate = _.max(cycleDates);
            if (minCycleDate && maxCycleDate) {
                newReportPeriod = [minCycleDate.toDate(), maxCycleDate.toDate()];
            }
        }
        setSelectedCycles(cycles);
        setReportPeriod(newReportPeriod);
        console.log("setReportPeriod", newReportPeriod)
    }


    /**
     * Set Report period and close the calendar if both values in the range are selected.
     * @param {Array} value - Array of date objects
     */
    function setTheReportPeriod(value = []) {
        setReportPeriod(value);
        if (value && value.length === 2 && value[1]) {
            document.getElementsByClassName('p-datepicker')[0].style.display = "none";
        }
    }

    /**
     * Function to generate the PDF of the report.
     */
    const downloadPDF = async () => {
        setIsDownloading(true);
        let reportDivs = document.getElementsByClassName('report-div');
        // Create a PDF document with landscape orientation, 1st report div width and the max height of all report divs in pixels
        const pdf = new jsPDF('l', 'px', [(reportDivs[0].clientWidth + 100), (reportDivs[0].clientHeight + 100)]);
        let pageIndex = 0;
        // Draw each report in canvas and create image of the canvas before saving the pdf
        for (const reportName of _.keys(ReportBusiness.REPORT_VARIABLE_MAP)) {
            // Create page for each project
            if (pageIndex > 0) {
                pdf.addPage([reportDivs[0].clientWidth + 100, reportDivs[pageIndex].clientHeight + 100], pageIndex === 6 ? 'p' : 'l');
            }
            const reportElement = document.getElementById(`${reportName}`);
            if (reportElement == null) {
                console.log("empty", reportName);
            }
            const options = {
                foreignObjectRendering: true,
                width: reportElement.clientWidth,
                height: reportElement.clientHeight
            };
            const projectCanvas = await html2canvas(reportElement, options);
            pdf.addImage(projectCanvas.toDataURL('image/jpeg'), 'JPEG', 50, 50, reportDivs[pageIndex].clientWidth, reportDivs[pageIndex].clientHeight);
            pageIndex++;
        }
        // To open the generated PDF in new window
        // pdf.output('dataurlnewwindow');
        const dateForRepordPeriod = `${moment(reportPeriod[0]).format("DDMMMYYYY")}-${moment(reportPeriod[1]).format("DDMMMYYYY")}`;
        pdf.save(`Cycle_${dateForRepordPeriod}.pdf`);
        setIsDownloading(false);

    }

    const csvDialogContent = () => {
        let currentReportNames = [...shadowReportNames]
        console.log("currentReportNames", currentReportNames)
        console.log("shadowReportNames", shadowReportNames)
        let allReports = _.values(ReportBusiness.REPORT_VARIABLE_MAP);
        allReports = _.uniq(allReports.slice(1));
        const repWithData = _.keys(cyclesReportData[0]?.data) || [];
        let csvOptions = [];

        for (const report of allReports) {
            let csvOption = { name: _.invert(ReportBusiness.REPORT_VARIABLE_MAP)[report], value: report };
            csvOption.disabled = repWithData.indexOf(report) < 0;
            csvOption.checked = currentReportNames?.indexOf(report) >= 0;
            csvOptions.push(csvOption);
        }

        return (
            <div className="fluid csv-options-dlg-content">
                <div className="col-lg-12" style={{ padding: '5px' }}>
                    <Checkbox inputId={`cb-all`} value="all"
                        checked={reportNames?.length === allReports.length}
                        onChange={() => selectAllCSVOptions()}></Checkbox>
                    <label htmlFor={`cb-all`} className="p-checkbox-label checkbox-label">All</label>
                </div>
                {csvOptions.map((item, index) => (
                    <div className="col-lg-12" style={{ padding: '5px' }} key={item.name}>
                        <Checkbox inputId={`cb-${index}`} value={item.name}
                            checked={item.checked} disabled={item.disabled}
                            onChange={() => {
                                console.log(`Changing CSV Option for ${item.name} - ${!item.checked}`);
                                changeCSVOption(item.value, !item.checked)
                            }}></Checkbox>
                        <label htmlFor={`cb-${index}`} className={`p-checkbox-label checkbox-label${item.disabled ? '-disabled' : ''}`}>{item.name}</label>
                    </div>
                ))}
            </div>
        );
    }

    /**
     * Function called when report is selected or unselected from the list
     * @param {string} reportName - name of the report passed from the checkbox component of the list dialog
     * @param {boolean} selected - to include or exclude
     */
    const changeCSVOption = (reportName, selected) => {

        console.log("Before set ReportNames", reportNames, "set for", reportName, "selected", selected, shadowReportNames);
        let tempReportNames = [...shadowReportNames];
        tempReportNames = tempReportNames.filter(rep => rep !== reportName);
        if (selected) {
            tempReportNames = [...tempReportNames, reportName];
        }
        console.log("After  set ReportNames", reportNames, "set for", reportName, "selected", selected, shadowReportNames);

        setReportNames(tempReportNames);
        shadowReportNames = tempReportNames

        return tempReportNames;
    }

    /**
     * Select or Unselect all reports in the dialog
     */
    const selectAllCSVOptions = () => {
        let repWithData = _.keys(visualReportData);
        if (repWithData.length > reportNames?.length) {
            setReportNames([...repWithData]);
            shadowReportNames = [...repWithData]
        } else {
            setReportNames([]);
            shadowReportNames = [];

        }
    }


    /**
     * To show the dialog with reports to be selected or unselected for downloading as CSV files.
     * @returns 
     */
    const showCSVReportDialog = () => {


        let newReportNames = shadowReportNames
        if (newReportNames.length === 0) {
            let allkeys = Object.keys(cyclesReportData[0].data);
            newReportNames = allkeys;
        }
        if (newReportNames.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'No Data', detail: 'None of the report has data to download' });
            return;
        }
        setReportNames([...newReportNames]);
        shadowReportNames = [...newReportNames];

        let dialog = { header: "CSV Report(s) to download", width: "40vw", actions: null, showIcon: false };
        dialog.detail = "Following selected reports will be downloaded. Report(s) with no data are disabled. Select/Unselect the reports of your interest and click 'Ok'";
        dialog.content = csvDialogContent;
        dialog.onSubmit = () => downloadCSV;
        dialog.actions = [{ id: 'yes', title: 'Ok', callback: downloadCSV },
        { id: 'no', title: 'Cancel', callback: closeDialog }]
        setCsvDlgVisible(true);
        setDialog(dialog);

    }

    /**
     * Closes the dialog.
     */
    const closeDialog = () => {
        setCsvDlgVisible(false);
    }

    /**
     * Function to download the report data in CSV format
     */
    const downloadCSV = () => {
        if (shadowReportNames?.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'No Report Selected', detail: 'No report selected to download' });
        }

        let reportData = ReportBusiness.generateExportData(cyclesReportData);
        // For every sub-report of the main report get report data
        for (const reportName of shadowReportNames) {
            let csvConfig = {};
            let csvList = [];            //Data list to export
            let colHeaders = [];        //Column header to export. Both no of headers and data values should be same.
            let isColHeaderSet = false; //Flag to identify if colHeader is added
            // Get the whole report data set from MainReport component
            const subReportData = reportData?.[reportName];
            subReportData?.map((data) => {
                let csvData = {};
                const colKeys = _.keys(data);
                // For every column of the data, replace the column name with the column title
                for (const colKey of colKeys) {
                    let colHeader = _.upperFirst(colKey);
                    if (!isColHeaderSet) {
                        colHeaders.push(colHeader);
                    }
                    csvData[colHeader] = data[colKey];
                }
                csvList.push(csvData);
                isColHeaderSet = true;
                return data;
            });
            // Pass column headers to CSV parser
            if (colHeaders.length > 0) {
                csvConfig.columns = colHeaders;
            }
            // Parse the CSV list and header opject to create CSV string
            const csvString = Papa.unparse(csvList, csvConfig);
            // Create a CSV BLOB from the csvString
            const blob = new Blob([csvString], { type: "text/csv" });
            // Add dummy link to click and download the created CSV

            let a = document.createElement("a");
            document.body.appendChild(a);
            a.style = "display: none";
            let url = window.URL.createObjectURL(blob);
            a.href = url;
            const currentReportPeriod = `${moment(reportPeriod[0]).format("DDMMMYYYY")}-${moment(reportPeriod[1]).format("DDMMMYYYY")}`;
            a.download = `Cycle_${reportName}_${currentReportPeriod}.csv`;
            a.click();
            window.URL.revokeObjectURL(url);
        }
        setCsvDlgVisible(false);
    }

    const LoadReportCycle = async (requestingCycles = selectedCycles) => {
        const cycleNames = _.map(_.orderBy(requestingCycles, 'start'), 'name');
        console.log("LoadReportCycle :setTheReportCycles", cycleNames, "isLoading", isLoading);
        let newCyclesReportData = [];
        let missingCycles = false;
        newCyclesReportData = _.values(await ReportService.getCyclesReport(cycleNames));
        newCyclesReportData.map(cycleReportData => {

            if (cycleReportData.data === "Your report is getting prepared. Please check again in a few minutes") {
                setProgressDots(prevDots => prevDots + '.');
                missingCycles = true;
                console.log("received no cycleReportData due to :", cycleReportData);
                return; // when we are still loading, exit.
            }
            console.log("received cycleReportData", cycleReportData);
            cycleReportData["overview"] = cycleReportData.projects_summary;


            return cycleReportData;
        });


        if (missingCycles) {
            console.log("There where missing Cycles, Retrying in 500ms")
            setTimeout(() => {
                LoadReportCycle();
            }, 500);
        } else {
            setCyclesReportData(newCyclesReportData);
            let reportData = ReportBusiness.generateVisualData(newCyclesReportData);
            setVisualReportData(reportData);
            setReportCycles(_.orderBy(_.cloneDeep(selectedCycles), 'start'));
            setIsLoading(false)
            console.log("Load Report Cycle Completed);")
        }
    }
    /**
     * Function called to set report cycles from the selected cycles.
     */
    const setTheReportCycles = async () => {
        let isNowLoading = true;
        setIsLoading(isNowLoading)
        setCyclesReportData([]);
        setReportNames([]);
        shadowReportNames = [];
        setReportCycles([]);
        setProgressDots(".");
        LoadReportCycle();
    }



    /**
     * Clears the report filters and report divs
     */
    const clearAll = () => {
        setCyclesReportData({});
        setSelectedCycles([]);
        setReportPeriod([]);
        setReportCycles([]);
    }

    return (
        <React.Fragment>
            {pageLoading &&
                <AppLoader />}
            {!pageLoading &&
                <div className="report-toolbar p-grid" style={{ marginTop: "10px", paddingBottom: "10px", borderBottom: "1px solid lightgrey" }}>
                    <label htmlFor="cycle" className="col-lg-1 col-md-2 col-sm-12">Cycle </label>
                    <AutoComplete className="col-lg-4 col-md-4 col-sm-12" multiple forceSelection autoFocus autoHighlight
                        tooltip="Type few letters of the cycle and select the cycle suggested" tooltipOptions={{ position: "top" }}
                        value={selectedCycles} suggestions={suggestedCycles}
                        completeMethod={searchCycles} field="name" onChange={(e) => { selectCycles(e.value) }} />
                    <label htmlFor="period" className="col-lg-2 col-md-2 col-sm-12" style={{ paddingRight: "0px" }}>For Period </label>
                    <div className="col-lg-3 col-md-3 col-sm-12 report-calendar">
                        <Calendar value={reportPeriod} selectionMode="range" dateFormat="yy-mm-dd"
                            tooltip="Select the date range for the report"
                            showButtonBar onChange={e => setTheReportPeriod(e.value)}></Calendar>
                    </div>
                    <div className="ReportButtonArray">
                        <Button className='ReportButton' icon="pi pi-check" tooltip="Generate Report"
                            onClick={() => setTheReportCycles()} disabled={selectedCycles?.length === 0 || reportPeriod?.length !== 2} />
                        <Button icon="pi pi-undo" tooltip="Clear All" className='ReportButton'
                            onClick={() => clearAll()} style={{ marginLeft: "10px" }} />


                        {reportCycles?.length > 0 &&
                            <>
                                <Button to={{}} className='ReportButton'
                                    title="Download Report Data in CSV format. Each report data will be downloaded as separate .csv file. Enable auto downloading multiple files in browser settings."
                                    onClick={() => showCSVReportDialog()} icon="pi pi-file-export" />

                                <Button title="Download Report as PDF" onClick={() => downloadPDF()} icon="pi pi-file-pdf" className='ReportButton' />
                                <Button title="Print Report" onClick={() => handlePrint()} icon="pi pi-print" className='ReportButton' />
                            </>
                        }
                    </div>
                </div>
            }
            {(isLoading || isDownloading) && <><div><ProgressBar mode="indeterminate" style={{ height: '5px' }} /></div> {progressDots}</>}

            {(reportCycles.length > 0) &&
                <div ref={reportComponentRef}>
                    <CycleReportContent ReportData={visualReportData} reportCycles={reportCycles} reportPeriod={reportPeriod} ></CycleReportContent>
                </div>
            }
            {csvDlgVisible &&
                <CustomDialog type="confirmation" visible={csvDlgVisible}
                    header={dialog.header} message={dialog.detail} actions={dialog.actions}
                    content={dialog.content} width={dialog.width} showIcon={dialog.showIcon}
                    onClose={() => closeDialog} onCancel={() => closeDialog} onSubmit={() => dialog.onSubmit} />
            }
        </React.Fragment>
    );

})
CycleReportMain.displayName = 'CycleReportMain';
export default CycleReportMain;

