import React, { Component } from 'react';
import { Link } from 'react-router-dom';
import moment from 'moment';
import _ from 'lodash';
import 'chart.js/auto';
import { Chart, registerables } from 'chart.js';
import { Bar } from 'react-chartjs-2';
import annotationPlugin from 'chartjs-plugin-annotation';

import ProjectService from '../../services/project.service';
import ReportService from '../../services/report.service';
import UnitConverter from '../../utils/unit.converter';
import UIConstants from '../../utils/ui.constants';

Chart.register(...registerables);
Chart.register(annotationPlugin);

/**
 * Component to get report data and display them for a Project.
 */
class ProjectReport extends Component {
    
    constructor(props) {
        super(props);
        this.state = {
            project: null,
            reportData: null,
            projectResources: {},
            suStatsList: [],
            resourceUtilization: []
        };
    }

    componentDidMount() {
        if (this.props.project) {
            this.loadProjectReport(this.props.project);
        }
    }

    componentDidUpdate() {
        if (this.state.project !== this.props.project) {
            this.loadProjectReport(this.props.project);
        }
    }

    /**
     * Get project report data and format or calculate values as required for the report.
     * @param {Object} project 
     */
    async loadProjectReport(project) {
        const resourceList = this.props.resourceList;
        const projectReport = await ReportService.getProjectReport(project.name);
        project.friend = (await ProjectService.getFriendsOfProject(project.name)).join(", ");
        let projectResources = {};
        if (projectReport.error) {
            console.log(projectReport.error);
        }   else {
            // Get all project resources and convert the values to display values
            for (let quota of projectReport.quota) {
                const resource = _.find(resourceList, ["name", quota.resource_type_id]);
                const conversionFactor = UnitConverter.resourceUnitMap[resource.quantity_value]?UnitConverter.resourceUnitMap[resource.quantity_value].conversionFactor:1;
                quota.convertedValue =  (quota.value / conversionFactor).toFixed(2);
                projectResources[quota.resource_type_id] = quota;
            }
        }
        let suStatsList = [], resourceUtilization = [];
        // Calculate SUB resource values from project resource valies.
        // If it is received from report API, this needs to be modified.
        if (projectReport["SUBs"]) {
            const projectObservingTime = projectResources["LOFAR Observing Time"]?projectResources["LOFAR Observing Time"].value:0;
            const projectProcessTime = projectResources["CEP Processing Time"]?projectResources["CEP Processing Time"].value:0;
            const projectLTAStorage = projectResources["LTA Storage"]?projectResources["LTA Storage"].value:0;
            const timeFactor = UnitConverter.resourceUnitMap["time"].conversionFactor;
            const dataSizeFactor = UnitConverter.resourceUnitMap["bytes"].conversionFactor;
            const toDisplayedPercent = function(value, total) {return ((value || 0) / total * 100).toFixed(2);}
            const toDisplayedValue = function (value, factor) {return ((value || 0) / factor).toFixed(2);}
            const toDisplayedSize = function(value) {return toDisplayedValue(value, dataSizeFactor);}
            const toDisplayedTime = function(value) {return toDisplayedValue(value, timeFactor);}
            let totalSUBObsTime = 0, totalProcessTime = 0, totalLTAStorage = 0;
            for (const subStatus of _.keys(projectReport["SUBs"])) {
                let subs = projectReport["SUBs"][subStatus];
                for (const sub of subs) {
                    let reportSub = _.cloneDeep(sub);
                    reportSub.acceptance_status = subStatus;
                    suStatsList.push(reportSub);
                }
            }
            suStatsList = _.orderBy(suStatsList, ['stop']);
            for (const reportSub of suStatsList) {
                if (reportSub.observed_duration) {
                    reportSub.observingTime = toDisplayedTime(reportSub.observed_duration);
                    if (reportSub.acceptance_status !== 'failed'){
                        totalSUBObsTime += reportSub.observed_duration;
                    }
                    reportSub.observingTimeInc = toDisplayedTime(totalSUBObsTime);
                    reportSub.observingTimeLeft = toDisplayedTime(projectObservingTime - totalSUBObsTime);
                    reportSub.observingTimeIncPercent = toDisplayedPercent(totalSUBObsTime, projectObservingTime);
                }
                if (reportSub.CPU_time_used_duration) {
                    reportSub.processTime = toDisplayedTime(reportSub.CPU_time_used_duration);
                    if (reportSub.acceptance_status !== 'failed'){
                        totalProcessTime += reportSub.CPU_time_used_duration;
                    }
                    reportSub.processTimeInc = toDisplayedTime(totalProcessTime);
                    reportSub.processTimeLeft = toDisplayedTime(projectProcessTime - totalProcessTime);
                    reportSub.processTimeIncPercent = toDisplayedPercent(totalProcessTime, projectProcessTime);
                }
                reportSub.ingestDataSize = toDisplayedSize(reportSub["ingested_data_size"]);
                if (reportSub.acceptance_status !== 'failed'){
                    totalLTAStorage += reportSub["ingested_data_size"];
                }
                reportSub.ingestDataIncPercent = toDisplayedPercent(totalLTAStorage, projectLTAStorage);
                reportSub.ingestDate = reportSub.ingested_date?moment.utc(reportSub.ingested_date).format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT):"";
                reportSub.execDate = moment.utc(reportSub.start).format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT);
                reportSub.observationSASId = reportSub["SAS ID"]["observation control"].join();
                reportSub.pipelinseSASId = (reportSub["SAS ID"]["preprocessing pipeline"].concat(reportSub["SAS ID"]["pulsar pipeline"])).join(", ");
            }
            let observTimeUtilization = {type: 'Observing', value: toDisplayedTime(projectReport.durations.total_observed),
                                         percent: toDisplayedPercent(totalSUBObsTime, projectObservingTime),
                                         succeeded: toDisplayedPercent(projectReport.durations.total_observed_succeeded, projectObservingTime),
                                         succeededValue: toDisplayedTime(projectReport.durations.total_observed_succeeded),
                                         acceptancePending: toDisplayedPercent(projectReport.durations.total_observed_acceptance_pending, projectObservingTime),
                                         acceptancePendingValue: toDisplayedTime(projectReport.durations.total_observed_acceptance_pending),
                                         failed: toDisplayedPercent(projectReport.durations.total_observed_failed, projectObservingTime),
                                         failedValue: toDisplayedTime(projectReport.durations.total_observed_failed),
                                         unit: 'hrs'};
            resourceUtilization.push(observTimeUtilization);
            let processTimeUtilization = {type: 'CEP Processing', value: toDisplayedTime(projectReport.processing_resources.CPU_time_used),
                                          percent: toDisplayedPercent(totalProcessTime, projectProcessTime),
                                          succeeded: toDisplayedPercent(projectReport.processing_resources.CPU_time_used_succeeded, projectProcessTime),
                                          succeededValue: toDisplayedTime(projectReport.processing_resources.CPU_time_used_succeeded),
                                          acceptancePending: toDisplayedPercent(projectReport.processing_resources.CPU_time_used_acceptance_pending, projectProcessTime),
                                          acceptancePendingValue: toDisplayedTime(projectReport.processing_resources.CPU_time_used_acceptance_pending),
                                          failed: toDisplayedPercent(projectReport.processing_resources.CPU_time_used_failed, projectProcessTime),
                                          failedValue: toDisplayedTime(projectReport.processing_resources.CPU_time_used_failed),
                                          unit: 'hrs'};
            resourceUtilization.push(processTimeUtilization);
            let ltaStorageUtilization = {type: 'LTA Storage', value: toDisplayedSize(projectReport.LTA_dataproducts.size),
                                         percent: toDisplayedPercent(totalLTAStorage, projectLTAStorage),
                                         succeeded: toDisplayedPercent(projectReport.LTA_dataproducts.size_succeeded, projectLTAStorage),
                                         succeededValue: toDisplayedSize(projectReport.LTA_dataproducts.size_succeeded),
                                         acceptancePending: toDisplayedPercent(projectReport.LTA_dataproducts.size_acceptance_pending, projectLTAStorage),
                                         acceptancePendingValue: toDisplayedSize(projectReport.LTA_dataproducts.size_acceptance_pending),
                                         failed: toDisplayedPercent(projectReport.LTA_dataproducts.size_failed, projectLTAStorage),
                                         failedValue: toDisplayedSize(projectReport.LTA_dataproducts.size_failed),
                                         unit: 'TB'};
            resourceUtilization.push (ltaStorageUtilization);
        }
        if (this.props.passReportData) {
            this.props.passReportData(project.name, {reportData: projectReport, projectResources: projectResources, 
                suStatsList: suStatsList, resourceUtilization: resourceUtilization});
        }
        const resourceUtilizationRates = resourceUtilization.map(resource => {
            const utilizationRate = (parseFloat(resource.succeeded||0) + parseFloat(resource.acceptancePending||0) + parseFloat(resource.failed||0)).toFixed(2);
            if (isNaN(utilizationRate) || !isFinite(utilizationRate)){
                return 0;
            }
            return utilizationRate;
        });
        resourceUtilizationRates.push(100);
        this.setState({project: project, reportData: projectReport, projectResources: projectResources,
                        suStatsList: suStatsList, resourceUtilization: resourceUtilization,
                        maxUtilization : Math.max.apply(null, resourceUtilizationRates)});
        
    }

    getTargetData() {
        const projectObservingTime = this.state.projectResources["LOFAR Observing Time"]?this.state.projectResources["LOFAR Observing Time"].value:0;
        const timeFactor = UnitConverter.resourceUnitMap["time"].conversionFactor;
        const sapsData = this.state.reportData["SAPs_exposure"];
        const targets = _.keys(sapsData);
        let targetReports = [];
        for (let target of targets) {
            let targetReport = {name: target};
            targetReport.exposure = parseFloat(sapsData[target] / timeFactor).toFixed(2);
            targetReport.expPercent = parseFloat(sapsData[target] / projectObservingTime * 100 ).toFixed(2);
            targetReports.push(targetReport);
        }
        const maxExposure = Math.max.apply(null,targetReports.map(function(obj) { return obj.exposure;}));
        let targetBarData = {
            labels: _.map(targetReports, 'name'),
            datasets: [
                {   label: 'Exposure', 
                    data: _.map(targetReports, 'exposure'),
                    backgroundColor: [
                        '#44a3ce',
                        '#44a3ce',
                        '#44a3ce'
                    ]
                }
            ]
        };
        let targetBarOptions = {
            indexAxis: 'y',
            elements: {
                bar: { borderWidth: 1 },
            },
            scales: {
                x: { max: maxExposure }
            },
            responsive: true,
            plugins: {
                legend: { position: 'right' },
                title: {
                    display: true,
                    text: 'Target Exposure',
                },
                tooltip: {
                    callbacks: {
                        label: function(tooltipItem) {
                            const itemData = _.find(targetReports, ['name', tooltipItem.label]);
                            return `${itemData?itemData.exposure:""} hrs`;
                        }
                    }
                }
            }
        };
        return {barData: targetBarData, barOptions: targetBarOptions};
    }

    /**
     * Renders the SU table header component
     * @returns Component
     */
    renderSUTableHeader() {
        return (
            <thead><tr>
            {
                this.props.SU_DETAILS_COLUMNS.map((item, index) =>  {
                    return <th key={("th_"+index)}>{item.headerTitle}</th>
                })
            }
            </tr></thead>
        );
    }

    /**
     * Renders SUB details rows
     * @returns Component
     */
    renderSURows() {
        const suStatsList = this.state.suStatsList;
        return (
            <>
             {
                 suStatsList.map((rowData, rowIndex) => {
                     return (
                         <tr key={("tr_"+rowIndex)} style={{border: "1px solid"}}>
                            { this.props.SU_DETAILS_COLUMNS.map((column, colIndex) => {
                                 return (
                                    <td key={(rowIndex+"_td_"+colIndex)}>
                                        {column.propertyName === "name"?(
                                        <Link to={`/schedulingunit/view/blueprint/${rowData['id']}`} target="_blank">{rowData[column.propertyName]}</Link>
                                        ):rowData[column.propertyName]}
                                    </td>
                                 )
                              })
                            }
                         </tr>
                     )
                 })
             }
            </>
        );
    }

    render() {
        const reportData = this.state.reportData;
        let barData = {}, barOptions = {};
        const resourceUtilization = this.state.resourceUtilization;
        // _.map(resourceUtilization, resource => {
        //     if (resource.type === "Observing") {
        //         resource.succeeded = 50;
        //         resource.succeededValue = 10;
        //         resource.failed = 25;
        //         resource.failedValue = 2.5
        //     }
        //     return resource;
        // })
        const targetData = reportData?this.getTargetData():null;
        // Resource Utilization bar chart data set with options
        if (resourceUtilization.length > 0) {
            barData = {
                labels: _.map(resourceUtilization, 'type'),
                datasets: [
                    {   label: 'Succeeded / Utilized', 
                        data: _.map(resourceUtilization, 'succeeded'),
                        backgroundColor: '#44a3ce'
                    },
                    {   label: 'Acceptance Pending',
                        data: _.map(resourceUtilization, 'acceptancePending'),
                        backgroundColor: '#A5A5A5'
                    },
                    {   label: 'Failed', 
                        data: _.map(resourceUtilization, 'failed'),
                        backgroundColor: '#ED7D31'
                    }
                ]
            };

            barOptions = {
                indexAxis: 'y',
                // Elements options apply to all of the options unless overridden in a dataset
                // In this case, we are setting the border of each horizontal bar to be 1px wide
                elements: {
                  bar: {
                    borderWidth: 1,
                  },
                },
                scales: {
                    x: {
                        stacked: true,
                        max: this.state.maxUtilization,
                        categoryPercentage: 1.0,
                        barPercentage: 1.0
                    },
                    y: {
                        stacked: true
                    }
                },
                responsive: true,
                plugins: {
                  legend: {
                    position: 'bottom',
                  },
                  title: {
                    display: true,
                    text: 'Resource Utilization (%)',
                  },
                  tooltip: {
                    callbacks: {
                        label: function(tooltipItem ) {
                            const itemData = _.find(resourceUtilization, ['type', tooltipItem.label]);
                            if (tooltipItem.dataset.label === "Succeeded / Utilized") {
                                return `${itemData?itemData.succeeded:""}% (${itemData?itemData.succeededValue:""} ${itemData?itemData.unit:""}) Succeeded`;
                            }
                            if (tooltipItem.dataset.label === "Acceptance Pending") {
                                return `${itemData?itemData.acceptancePending:""}% (${itemData?itemData.acceptancePendingValue:""} ${itemData?itemData.unit:""}) Acceptance Pending`;
                            }
                            if (tooltipItem.dataset.label === "Failed") {
                                return `${itemData?itemData.failed:""}% (${itemData?itemData.failedValue:""} ${itemData?itemData.unit:""}) Failed`;
                            }
                        }
                    }
                  },
                  annotation: {
                    annotations: [{
                        type: 'box',
                        drawTime: "beforeDatasetsDraw",
                        xScaleID: 'x',
                        yScaleID: 'y',
                        xMin: 100,
                        xMax: this.state.maxUtilization,
                        borderWidth: 0,
                        backgroundColor: 'rgb(245, 155, 155)',
                        label: {
                          enabled: false,
                          content: ''
                        }
                    }]
                  }
                }
            };
        }
        return(
            <React.Fragment>
                {reportData &&
                <>
                <div className="report-div" id={`${this.props.project.name}-report-div`}>
                    <div id={`${this.props.project.name}-project-details`}>
                    <h2 style={{textAlign: "center", marginBottom:"25px"}}>Report statistics for project {this.props.project.name}</h2>
                    <div className="p-grid report-wrap">
                        <div className="col-lg-3 col-md-4 col-sm-12">
                            <label>Project Documentation</label>
                        </div>
                        <div className="col-lg-9 col-md-8 col-sm-12">
                            <a href="https://support.astron.nl/jira" target="_blank" rel="noreferrer">Link to Jira Ticket</a>
                        </div>
                        <div className="col-lg-3 col-md-4 col-sm-12">
                            <label>Project statistics over the period</label>
                        </div>
                        <div className="col-lg-8 col-md-9 col-sm-12">
                            <span>{this.props.reportPeriod?`${moment(this.props.reportPeriod[0]).format("MMM DD YYYY")} - ${moment(this.props.reportPeriod[1]).format("MMM DD YYYY")}`:"-"}</span>
                        </div>
                        <div className="col-lg-3 col-md-4 col-sm-12">
                            <label>Contact Project Friend</label>
                        </div>
                        <div className="col-lg-8 col-md-9 col-sm-12">
                            <span>{this.props.project.friend || "..."}</span>
                        </div>
                        <div className="col-lg-3 col-md-4 col-sm-12">
                            <label>Awarded Observing Time(hours)</label>
                        </div>
                        <div className="col-lg-8 col-md-9 col-sm-12">
                            <span>{this.state.projectResources["LOFAR Observing Time"]?this.state.projectResources["LOFAR Observing Time"].convertedValue:"-"}</span>
                        </div>
                        <div className="col-lg-3 col-md-4 col-sm-12">
                            <label>Awarded Processing Time(hours)</label>
                        </div>
                        <div className="col-lg-8 col-md-9 col-sm-12">
                            <span>{this.state.projectResources["CEP Processing Time"]?this.state.projectResources["CEP Processing Time"].convertedValue:"-"}</span>
                        </div>
                        <div className="col-lg-3 col-md-4 col-sm-12">
                            <label>Awarded LTA Storage(TB)</label>
                        </div>
                        <div className="col-lg-8 col-md-9 col-sm-12">
                            <span>{this.state.projectResources["LTA Storage"]?this.state.projectResources["LTA Storage"].convertedValue:"-"}</span>
                        </div>
                        <div className="col-lg-3 col-md-4 col-sm-12">
                            <label>Awarded Support Time(hours)</label>
                        </div>
                        <div className="col-lg-8 col-md-9 col-sm-12">
                            <span>{this.state.projectResources["LOFAR Support Time"]?this.state.projectResources["LOFAR Support Time"].convertedValue:"-"}</span>
                        </div>
                    </div>
                    {resourceUtilization.length > 0 &&
                    <div className="resource-utilization" id={`${this.props.project.name}-resource-utilization`}
                        style={{paddingTop: "10px", paddingBottom: "10px"}}>
                        <Bar data={barData} options={barOptions} plugins={annotationPlugin} width="50%" height="10"/>
                    </div>
                    }
                    </div>
                    <div className="su-details" id={`${this.props.project.name}-su-details`}>
                        <label>Scheduling Units of the project</label>
                        <table className="report-table">
                            {this.renderSUTableHeader()}
                            <tbody>
                                {this.renderSURows()}
                            </tbody>
                        </table>
                    </div>
                    {targetData &&
                    <div className="target-exposure" id={`${this.props.project.name}-target-exposure`}
                        style={{paddingTop: "10px", paddingBottom: "10px"}}>
                        <Bar data={targetData.barData} options={targetData.barOptions} width="50%" height="10"/>
                    </div>
                    }
                </div>
                </>
                }
            </React.Fragment>
        );
    }

}

export default ProjectReport;