import moment from "moment";
import UnitConversion from "../../utils/unit.converter";
import ScheduleService from "../../services/schedule.service";
import {appGrowl} from "../../layout/components/AppGrowl";

/**
 * Checks if a property is present and if it has a valid value. If the value is invalid but the property is present, it is deleted
 * @param propertyNames list<string>
 * @param constraintTemplate
 * @return {*} updated constraintTemplate
 */
export function deleteInvalidProperties(propertyNames, constraintTemplate) {
    if (constraintTemplate && Object.keys(constraintTemplate).length > 0) { //template exists
        propertyNames.forEach(propertyName => {
            const isInvalidProperty = constraintTemplate.time.hasOwnProperty(propertyName) && !constraintTemplate.time[propertyName];
            if (isInvalidProperty) {
                delete constraintTemplate.time[propertyName]
            }
        })
    }
    return constraintTemplate
}

export function formatNestedTimeObject(timeArray) {
    const formattedTimeObj = {};
    for (let timeElement in timeArray) {
        if (timeArray[timeElement]) {
            formattedTimeObj[timeElement] = moment(timeArray[timeElement]).utc(true).format();
        }
    }
    return formattedTimeObj;
}

/**
 * Invalid time properties are deleted and valid (nested) time properties are formatted
 * @param constraintTemplate
 * @return {*} updated constraintTemplate
 */
export function cleanTimePropertiesOfConstraintTemplate(constraintTemplate) {
    if (constraintTemplate.time!==undefined) {
        constraintTemplate = deleteInvalidProperties(['at', 'after', 'before'], constraintTemplate)
        for (const type in constraintTemplate.time) {
            const timeElement = constraintTemplate.time[type];

            if (timeElement && timeElement.length) {
                if (typeof timeElement === 'string') {
                    constraintTemplate.time[type] = moment(timeElement).utc(true).format()
                } else {
                    constraintTemplate.time[type] = timeElement.map(timeArray => formatNestedTimeObject(timeArray))
                }
            }
        }
    }
    return constraintTemplate;
}

/**
 * Sky properties are converted (HHmmss to seconds + degrees to radians)
 * @param constraintTemplate
 * @return {{sky}|*} updated constraintTemplate
 */
export function cleanSkyPropertiesOfConstraintTemplate(constraintTemplate) {
    if (constraintTemplate.sky!==undefined) {
        if (constraintTemplate.sky.transit_offset) {
            constraintTemplate.sky.transit_offset.from = UnitConversion.getHHmmssToSecs(constraintTemplate.sky.transit_offset.from);
            constraintTemplate.sky.transit_offset.to = UnitConversion.getHHmmssToSecs(constraintTemplate.sky.transit_offset.to);
        }
        UnitConversion.degreeToRadians(constraintTemplate.sky);
    }
    return constraintTemplate;
}

/**
 * Editing the constraints via the summary editor pop-up, calls this function upon save (TODO: use everywhere)
 * It changes the human-readable constraint input so the API can accept it
 * - sky properties are converted (HHmmss to seconds + degrees to radians)
 * It then retrieves the correct SU blueprint and updates the constraint template.
 * The API response for updating (success/error) is reported back to the user???
 * @param suId
 * @param updatedConstraintTemplate the human-readable input of the constraint template
 * @param setOnClose callback function of the pop-up for closing
 * @param setOnSaveDisabled callback function of the pop-up for saving
 * @param setShowSummary callback function that closes the summary when constraints were successfully updated
 * @return {Promise<void>}
 */
export async function updateSUConstraintTemplate(suId, updatedConstraintTemplate, setOnClose, setOnSaveDisabled, setShowSummary) {
    setOnClose(false)
    setOnSaveDisabled(true)
    const timeCleanedConstraintTemplate = cleanTimePropertiesOfConstraintTemplate(updatedConstraintTemplate)
    const timeAndSkyCleanedConstrainTemplate = cleanSkyPropertiesOfConstraintTemplate(timeCleanedConstraintTemplate)
    let suBluePrint = await ScheduleService.getConstraintsEditorOutputService(suId);
    if (suId !== suBluePrint.id) {
        console.log("While editing, the reference blueprint changed. " +
            "It will overwrite the already changed version's constraints for id (original, updated):", suId, suBluePrint)
    }
    suBluePrint.scheduling_constraints_doc = timeAndSkyCleanedConstrainTemplate
    const response = await ScheduleService.setConstraintsEditorOutputService(suBluePrint);
    if (response.status !== 200) {
        appGrowl.show({severity: 'error', summary: 'Invalid constraints', detail: response.message});
    } else {
        //TODO: this is a quick fix to make sure the correct information will be shown in the summary
        // (include here: https://support.astron.nl/jira/browse/TMSS-2717)
        setShowSummary(false)
    }
}

/**
 * Combines constraint editing pop-up actions (close, save, cancel)
 * @param value boolean indicating if editing is done or not
 * @param setOnClose callback function of the pop-up for closing
 * @param setOnSaveDisabled callback function of the pop-up for saving
 * @param setCancelStatus callback function of the pop-up for cancelling
 */
export function editConstraintSettings(value, setOnClose, setOnSaveDisabled, setCancelStatus) {
    setOnClose(value)
    if (value) {
        setOnSaveDisabled(false)
        setCancelStatus(false)
    } else {
        setOnSaveDisabled(true)
        setCancelStatus(false)
    }
}