import React, { Component } from 'react';
import _ from 'lodash';

import { appGrowl } from '../../layout/components/AppGrowl';
import { RadioButton } from 'primereact/radiobutton';
import AppLoader from '../../layout/components/AppLoader';

import moment from 'moment';
import { Button } from 'primereact/button';
import Flatpickr from "react-flatpickr";
import UIConstants from '../../utils/ui.constants';
import ViewTable from '../../components/ViewTable';
import TaskService from '../../services/task.service';
import UtilService from '../../services/util.service';
import UnitConverter from '../../utils/unit.converter';
import "flatpickr/dist/flatpickr.css";
import { InputText } from 'primereact/inputtext';

/**
 * Component to create a new System Event
 */
export class AffectedTasksSelector extends Component {
    TABLE_NAME = "affected_tasks_list";
    TASK_BLUEPRINT_EXPAND = 'scheduling_unit_blueprint,scheduling_unit_blueprint.draft.scheduling_set,subtasks';
    TASK_BLUEPRINT_FIELDS = ['id', 'name', 'short_description', 'on_sky_start_time', 'on_sky_stop_time', 'scheduling_unit_blueprint.id', 
                             'scheduling_unit_blueprint.draft.scheduling_set.project_id', 'scheduling_unit_blueprint.name','subtasks.id','subtasks.primary'];
    constructor(props) {
        super(props);
        this.state ={
            searchBy : 'SUB',
            suId: '',
            start: null,
            stop: null,
            errors: {},
            validSearch: false,
            taskBlueprintlist: null,
            defaultcolumns: {
                on_sky_start_time: {name: "Observation Start Time",  format:UIConstants.CALENDAR_DATETIME_FORMAT},
                on_sky_stop_time: {name: "Observation End Time",  format:UIConstants.CALENDAR_DATETIME_FORMAT},
                scheduling_unit_blueprint_draft_scheduling_set_project_id: {name: "Project"},
                scheduling_unit_blueprint_id: {name: "Scheduling Unit Id"},
                scheduling_unit_blueprint_name: {name: "Scheduling Unit Name"},
                id:{name:"Task Id",},
                short_description: {name: "Short Description"},
                name:{name:"Name",},
                subtasks_0_id: {name: "Control Id"}
            },
            optionalcolumns:  [{}],
            columnclassname: [{
                "System Event Id":"filter-input-50",
            }],
            isLoading: false,
        }
        this.totalPage = 0;
        this.columnMap = [];
        this.defaultSortColumn =  [{ id: "Observation Start Time", desc: true }];
        this.pageUpdated = true;
        this.tooltipOptions = UIConstants.tooltipOptions;
        this.setSearchParam = this.setSearchParam.bind(this);
        this.onRowSelection = this.onRowSelection.bind(this);
        this.doSearch = this.doSearch.bind(this);
        this.checkSearch = this.checkSearch.bind(this);
        this.handleEvent = this.handleEvent.bind(this);
    }

    async componentDidMount() {
        await this.initSystemEvent();
    }

    /**
     * Initialize the system event details
     */
    async initSystemEvent() {
        const promises = [UtilService.getUTC(),];
        Promise.all(promises).then(async responses => {
            let systemTime = moment.utc(responses[6]);
            await this.setState({
                systemTime: systemTime,
                start: this.props.startTime,
                stop: this.props.stopTime,
                suId: this.props.dialogFilters.suId?this.props.dialogFilters.suId:this.props.suId,
                searchBy: this.props.dialogFilters.searchBy?this.props.dialogFilters.searchBy:this.state.searchBy,
                dialogFilters: this.props.dialogFilters,
            });
            this.checkSearch();
        });
    }

    /**
     * Set UI param
     * @param {String} key 
     * @param {String/Object} e 
     */
    async setSearchParam(key, e) {
        if (key === 'start' || key === 'stop' || key === 'searchBy') {
            await this.setState({
                [key]: e,
            });
        }   else {
            await this.setState({
                [key]: e.target.value,
            });
        }
        await this.checkSearch();
    }

    /**
     * Callback function passed to ViewTable component to pass back the selected rows.
     * @param {Array} selectedRows - Subset of data passed to the ViewTable component based on selection.
     */
    onRowSelection(selectedRows) {
        this.props.onRowSelection(selectedRows);
    }

    /** Function called while sorting table */
    toggleBySorting(sortData) {}

    /**
     * Fetch Task details for given search condition
     */
    async doSearch() {
        let filterQry = '';
        this.setState({loadingStatus:true});
        if ( this.state.searchBy === 'SUB') {
                let values = UnitConverter.getSubbandOutput(this.state.suId);
                if (values && values.length === 1) {
                    filterQry = 'scheduling_unit_blueprint='+values[0];
                }   else {
                    values = values.sort();
                    filterQry = 'scheduling_unit_blueprint_min='+values[0]+'&scheduling_unit_blueprint_max='+values[values.length-1];
                }
        }   else {
            filterQry = 'task_type=observation&';
            if (this.state.start) {
                if (this.state.start !== '') {
                    filterQry += 'on_sky_start_time_after='+ moment(new Date(this.state.start)).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";                
                }
                if (this.state.stop && this.state.stop !== '') {
                    filterQry += 'on_sky_start_time_before='+moment(new Date(this.state.stop)).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                }
            }
            if (this.state.stop) {
                if (this.state.start && this.state.start !== '') {
                    filterQry += 'on_sky_stop_time_after='+ moment(new Date(this.state.start)).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";                
                }
                if (this.state.stop !== '') {
                    filterQry += 'on_sky_stop_time_before='+moment(new Date(this.state.stop)).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                }
            }
            filterQry = filterQry.substring(0, filterQry.length-1);
        }
        filterQry += '&name=observation&ordering=id';
        await this.getTaskBlueprintList(filterQry);
        this.setState({loadingStatus:false});
    }

    /**
     * get Task for given query
     * @param {*} filterQry 
     */
    async getTaskBlueprintList(filterQry) {
        const taskCount = await TaskService.getTaskCount('blueprint');
        const response = await TaskService.getExpandedTasksWithFilter('blueprint', this.TASK_BLUEPRINT_EXPAND, this.TASK_BLUEPRINT_FIELDS, filterQry, '', taskCount, 0);
        this.pageUpdated = true;
        let taskList = [];
        if (response && response.data) {
            let taskRowObject = {};
            let taskRowlist = [];
            taskRowlist = await this.flattenObject( response.data.results, taskRowObject, null, taskRowlist);
            this.totalPage = response.data.count;
            taskList = taskRowlist.length>0?taskRowlist:[];
        }   else {
            appGrowl.show({severity: 'error', summary: 'Error', detail: 'Unable to fetch records'});
        }
        await this.setState({
            isLoading: false,
            taskBlueprintlist: taskList,
        });
    }
    
    /**
     * Flatten JSON array to json object
     * @param {JSON} jsonObject - json array value
     * @param {empty json} taskRowObject - json object
     * @param {String} prefixkey - parent key in prefix
     * @param {JSON Array} arrayObjList - Store the processed json array
     * @returns 
     */
    async flattenObject(jsonObject, taskRowObject, prefixkey, arrayObjList) {
        await jsonObject.map(item => {
            Object.keys(item).forEach(key => {
              if (item[key] && typeof item[key] === 'object') {
                Object.keys(item[key]).forEach(innerKey => {
                    if (item[key][innerKey] && typeof item[key][innerKey] === 'object') {
                        return this.flattenObject([item[key][innerKey]], taskRowObject, `${key}_${innerKey}`, arrayObjList)
                    }   else {
                        taskRowObject[(prefixkey && prefixkey !== ''?prefixkey+'_':'')+`${key}_${innerKey}`] = item[key][innerKey];
                    }
                });
              } else {
                if (taskRowObject) {
                    taskRowObject[(prefixkey && prefixkey !== ''?prefixkey+'_':'')+key] = item[key];
                }
              }
            });
            if (!arrayObjList) {
                arrayObjList = [];
            }
            const tmpObj = _.find(arrayObjList, {id: taskRowObject.id})
            if(!tmpObj){
                arrayObjList.push(_.cloneDeep(taskRowObject));
                taskRowObject = {}
            }
          });
        return arrayObjList;
    }

    /**
     * Clear the input field in UI
     */
    async clearSUBId() {
        await this.setState({
            suId: '',
        });
        let filters = this.props.dialogFilters;
        filters['suId'] = '';
        this.props.synchFilter(filters);
        await this.checkSearch();
    }

    /**
     * Enable/Disable the Search button in UI
     */
    async checkSearch(){
        let filters = this.state.dialogFilters;
        let validSearch = (this.state.searchBy === 'TASK' && ((this.state.start && this.state.start !== '') || (this.state.stop && this.state.stop !== ''))) 
        || (this.state.searchBy === 'SUB' && this.state.suId && this.state.suId !== '');
        filters['searchBy'] = this.state.searchBy;
        filters['start'] = this.state.start;
        filters['stop'] = this.state.stop;
        filters['suId'] = this.state.suId;
        await this.setState({
            validSearch: validSearch,
            dialogFilters: filters
        });
        this.props.synchFilter(filters);
    }

    /**
     * 
     * @param {Key Event} e - Key code
     */
    handleEvent(e) {
        var key = e.which || e.keyCode;
        if(key === 13 || key === 'Enter') {
            this.doSearch();
        }
    }

    render() {        
        return (
            <React.Fragment>
                <div className="searh_by p-field p-grid grouping" style={{padding: '2em,0.5em,0em,0.5em', height: 'auto'}} >
                    <fieldset>
                        <legend>
                            <label style={{marginLeft: "3px", marginRight: "3px"}}>Search By</label>
                        </legend> 
                        <div className="row affectedTask">
                            <div className="col-sm-3" style={{display: 'flex',}}>
                                <div className='row' >
                                    <div className="col-lg-8 col-sm-12" style={{display: 'flex',}}>
                                        <RadioButton value='SUB' name="SUB" onChange={() => this.setSearchParam('searchBy', 'SUB')}
                                            checked={this.state.searchBy === 'SUB'} tooltip="Search by Scheduling Unit Blueprint Id"/>
                                        <label style={{marginLeft: '0.25em'}}>Scheduling Unit</label>
                                    </div>
                                    <div className="col-lg-4 col-sm-12" style={{display: 'flex',}}>
                                        <RadioButton value='TASK' name="Task" onChange={() => this.setSearchParam('searchBy', 'TASK')}
                                            checked={this.state.searchBy === 'TASK'} tooltip="Search by Sky Start and End time of the Task"/>
                                        <label style={{marginLeft: '0.25em',}}>Time</label>
                                        </div>
                                </div>
                            </div>
                            <div className="col-sm-7">
                                {this.state.searchBy === 'SUB' ?
                                   <div className='row' style={{borderLeft: '1px solid lightgray', borderRight: '1px solid lightgray'}}>
                                        <div className="col-sm-3">
                                            <label>Scheduling Unit Id</label>
                                        </div>
                                        <div className="col-sm-8">
                                            <InputText  style={{height: '2em'}} keyfilter={/[\d\,\.]/}  validateOnly={false} placeholder="Enter Scheduling Unit Id(s)"
                                                onChange={(e) => this.setSearchParam('suId', e)}
                                                tooltip="Enter Scheduling Unit Id. To search multiple, enter ids separated by comma (1,2,3) and for range separated by 2 dots (1..10). Also can use both together (1,2,5..10)" tooltipOptions={this.tooltipOptions}
                                                value={this.state.suId}
                                                onKeyDown={this.handleEvent}/> 
                                        </div>
                                        <div className="col-sm-1">
                                            <a href="#"  style={{marginTop: '0.25em'}} onClick={() => this.clearSUBId()} title="Clear search field"> 
                                                <i className={"fas fa-sync-alt"} aria-hidden="true" />
                                            </a> 
                                        </div>  
                                    </div>
                                   :
                                   <div className='row' style={{borderLeft: '1px solid lightgray', borderRight: '1px solid lightgray', display: "grid"}}>
                                        <div className='row' style={{display: 'flex', marginLeft: '1em'}}>
                                            <div className="col-lg-5 col-sm-12"><label>Start Time</label></div>
                                            <div className="col-lg-1 col-sm-12"></div>
                                            <div className="col-lg-5 col-sm-12"><label>End Time</label></div>
                                        </div>
                                        <div className="row" style={{display: 'flex', marginLeft: '1em'}}>
                                            <div className='col-lg-5 col-sm-12'>
                                                <Flatpickr data-enable-time data-input 
                                                    options={{
                                                        "inlineHideInput": true,
                                                        "wrap": true,
                                                        "enableSeconds": true,
                                                        "time_24hr": true,
                                                        "minuteIncrement": 1,
                                                        "allowInput": true,
                                                        "defaultDate": this.state.systemTime.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT),
                                                        "defaultHour": this.state.systemTime.hours(),
                                                        "defaultMinute": this.state.systemTime.minutes()
                                                    }}
                                                    style={{width: '14em'}}
                                                    title="Choose the Sky start time"
                                                    value={this.state.start}
                                                    onChange={value => {
                                                        this.setSearchParam('start', value[0] ? value[0] : this.state.start);
                                                    }}
                                                    onKeyDown={this.handleEvent} >
                                                    <input type="text" data-input className={`p-inputtext p-component ${this.state.errors.start && this.state.touched.start ? 'input-error' : ''}`} />
                                                    <i className="fa fa-calendar" data-toggle style={{ position: "absolute", marginLeft: '-25px', marginTop: '5px', cursor: 'pointer' }} ></i>
                                                    <i className="pi pi-times" style={{ position: "absolute", marginLeft: '-50px', marginTop: '5px', cursor: 'pointer' }}
                                                        onClick={() => { this.setSearchParam('start', '');}}></i>
                                                </Flatpickr>
                                                {this.state.errors.start && this.state.touched.start &&
                                                <label className={this.state.errors.start && this.state.touched.start ? "error" : "info"}>
                                                    {this.state.errors.start && this.state.touched.start ? this.state.errors.start : ""}
                                                </label>}
                                            </div>
                                            <div className="col-lg-1 col-sm-12"></div>
                                            <div className='col-lg-5 col-sm-12'>
                                                <Flatpickr data-enable-time data-input 
                                                    options={{
                                                        "inlineHideInput": true,
                                                        "wrap": true,
                                                        "enableSeconds": true,
                                                        "time_24hr": true,
                                                        "minuteIncrement": 1,
                                                        "allowInput": true,
                                                        "minDate": this.state.start ? this.state.start.toDate : '',
                                                        "defaultDate": this.state.systemTime.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT),
                                                        "defaultHour": this.state.systemTime.hours(),
                                                        "defaultMinute": this.state.systemTime.minutes()
                                                    }}
                                                    style={{width: '14em'}}
                                                    title="Choose the Sky End time"
                                                    value={this.state.stop}
                                                    onChange={value => {
                                                        this.setSearchParam('stop', value[0] ? value[0] : this.state.stop);
                                                    }}
                                                    onKeyDown={this.handleEvent} >
                                                    <input type="text" data-input className={`p-inputtext p-component ${this.state.errors.stop && this.state.touched.stop ? 'input-error' : ''}`} />
                                                    <i className="fa fa-calendar" data-toggle style={{ position: "absolute", marginLeft: '-25px', marginTop: '5px', cursor: 'pointer' }} ></i>
                                                    <i className="pi pi-times" style={{ position: "absolute", marginLeft: '-50px', marginTop: '5px', cursor: 'pointer' }}
                                                        onClick={() => { this.setSearchParam('stop', '');}}></i>
                                                </Flatpickr>
                                            </div>
                                        </div>
                                    </div>
                                }
                            </div>
                            <div className="col-sm-2 p-col-1" style={{marginTop: '-0.5em'}}>
                                <Button label="Search" className="p-button-primary" icon="pi pi-search" onClick={this.doSearch}
                                disabled={!this.state.validSearch} tooltip="Click to search the Affected Task(s)" style={{right: 'unset !important'}}/>
                            </div>
                        </div>
                    </fieldset>
                </div>
                <div className={`p-field p-grid grouping`} style={{padding: '0em,0.5em,0em,0.5em'}} >
                    { this.state.isLoading ? <AppLoader /> :
                    <div style={{width: "100%"}}>
                        {this.state.taskBlueprintlist &&
                            <div>
                                <div style={{marginTop: '0.75em', marginBottom: '0.75em'}}>
                                    <span> Select the task(s) affected and click ‘Apply’</span>
                                </div>
                                <ViewTable 
                                    data={this.state.taskBlueprintlist} 
                                    totalPage={this.totalPage}
                                    defaultcolumns={[this.state.tmpDefaulcolumns ? this.state.tmpDefaulcolumns : this.state.defaultcolumns]} 
                                    optionalcolumns={this.state.tmpOptionalcolumns ? this.state.tmpOptionalcolumns : this.state.optionalcolumns}
                                    columnclassname={this.state.columnclassname}
                                    columnOrders={this.state.tmpColumnOrders}
                                    defaultSortColumn={this.defaultSortColumn}
                                    showaction={false}
                                    keyaccessor="id"
                                    tablename={this.TABLE_NAME}
                                    allowRowSelection={true}
                                    onRowSelection = {this.onRowSelection}
                                    toggleBySorting={(sortData) => this.toggleBySorting(sortData)}
                                    showGlobalFilter={false}
                                    pageUpdated={this.pageUpdated}
                                    storeFilter={false}
                                />
                            </div>
                        }
                    </div>
                }
                </div>
            </React.Fragment>
        );
    }
}

export default AffectedTasksSelector;