import {getIntervalRendererLST, getTimeSteps} from "./timeline.headers.helper";
import {render} from "@testing-library/react";
import {removeReact18ConsoleErrors} from "../../../utils/test.helper";

removeReact18ConsoleErrors()

describe('getTimeSteps', () => {
    test.each([1.5, 12, undefined])('should return default timesteps when duration is: %s', (duration) => {
        const result = getTimeSteps(duration);
        expect(result).toEqual({
            second: 15,
            minute: 5,
            hour: 0.5,
            day: 1,
            month: 1,
            year: 1
        });
    })

    it('should return correct time steps when duration is less than or equal to 0.5 hours', () => {
        const result = getTimeSteps(0.5);
        expect(result).toEqual({
            second: 15,
            minute: 1,
            hour: 0.5,
            day: 1,
            month: 1,
            year: 1
        });
    });

    it('should return correct time steps when duration is greater than or equal to 2 hours and less than 4 hours', () => {
        const result = getTimeSteps(3);
        expect(result).toEqual({
            second: 15,
            minute: 10,
            hour: 0.5,
            day: 1,
            month: 1,
            year: 1
        });
    });

    it('should return correct time steps when duration is greater than 4 hours and less than 12 hours', () => {
        const result = getTimeSteps(8);
        expect(result).toEqual({
            second: 15,
            minute: 5,
            hour: 0.5,
            day: 1,
            month: 1,
            year: 1
        });
    });
});

describe('getIntervalRendererLST', () => {
    it('should render "-" when lstShiftInSeconds is undefined', () => {
        const intervalProps = {
            getIntervalProps: jest.fn(() => ({})),
            intervalContext: {interval: {startTime: {add: jest.fn()}}},
        };
        const lstShiftInSeconds = undefined

        const content = render(getIntervalRendererLST({...intervalProps}, lstShiftInSeconds))

        expect(content.getByTestId("dateheader-lst--")).toBeInTheDocument()
    });

    it('should render formatted time when lstShiftInSeconds is defined', () => {
        const mockedAdd = jest.fn().mockReturnValue({format: jest.fn().mockReturnValue('12:34')});
        const startTime = {add: mockedAdd }
        // we need a clone function in startTime , since getIntervalRendererLST clones startTime ...
        startTime.clone = jest.fn().mockReturnValue(startTime); 
        const intervalProps = {
            getIntervalProps: jest.fn(() => ({})),
            intervalContext: {interval: {startTime }},
        };
        
        const lstShiftInSeconds = 3600

        const content = render(getIntervalRendererLST({...intervalProps}, lstShiftInSeconds))

        expect(content.getByTestId("dateheader-lst-12:34")).toBeInTheDocument()
        expect(mockedAdd).toHaveBeenCalledWith(3600, 'seconds');
    });
    
    it('should render formatted time when intervalContext is empty', () => {
        const intervalProps = {
            getIntervalProps: jest.fn(() => ({})),
            intervalContext: undefined,
            lstShiftInSeconds: 1337
        };
        const lstShiftInSeconds = 1337
        const content = render(getIntervalRendererLST({...intervalProps}, lstShiftInSeconds)) // the renderer validates input and does not longer throw an error
        expect(content.getByTestId("dateheader-lst--")).toBeInTheDocument()
    });
    

});